#include "PS2.h"

// LED IO口初始化
void PS2_ADC_Init(void)
{
	GPIO_InitTypeDef GPIO_InitStructure; // 定义结构体
	// 开启时钟
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);

	// X轴检测引脚
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN; // 模拟输入
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	// Y轴检测引脚
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN; // 模拟输入
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	// Z轴检测引脚
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_2;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU; // 输入上拉
	GPIO_Init(GPIOA, &GPIO_InitStructure);
}

/*
 *==============================================================================
 *函数名称：ADC1_Init
 *函数功能：初始化ADCx
 *输入参数：无
 *返回值：无
 *备  注：无
 *==============================================================================
 */
void ADC1_Init(void)
{
	// 结构体定义
	ADC_InitTypeDef ADC_InitStructure;

	// 开启时钟
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1, ENABLE);

	// 设置ADC分频因子6 72M/6=12,ADC最大时间不能超过14M
	RCC_ADCCLKConfig(RCC_PCLK2_Div6);

	// 规则通道配置
	ADC_RegularChannelConfig(ADC1, ADC_Channel_0, 0, ADC_SampleTime_239Cycles5);
	ADC_RegularChannelConfig(ADC1, ADC_Channel_1, 0, ADC_SampleTime_239Cycles5);

	// ADC参数配置
	ADC_InitStructure.ADC_Mode = ADC_Mode_Independent;					// 独立模式
	ADC_InitStructure.ADC_ScanConvMode = DISABLE;						// 非扫描模式
	ADC_InitStructure.ADC_ContinuousConvMode = DISABLE;					// 关闭连续转换
	ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_None; // 禁止触发检测，使用软件触发
	ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;				// 右对齐
	ADC_InitStructure.ADC_NbrOfChannel = 1;								// 1个转换在规则序列中 也就是只转换规则序列1
	ADC_Init(ADC1, &ADC_InitStructure);									// ADC初始化

	ADC_Cmd(ADC1, ENABLE); // 开启AD转换器

	// ADC校准
	ADC_ResetCalibration(ADC1); // 重置指定的ADC的校准寄存器
	while (ADC_GetResetCalibrationStatus(ADC1))
		; // 获取ADC重置校准寄存器的状态

	ADC_StartCalibration(ADC1); // 开始指定ADC的校准状态
	while (ADC_GetCalibrationStatus(ADC1))
		; // 获取指定ADC的校准程序
	ADC_SoftwareStartConvCmd(ADC1, ENABLE);
}

/*
 *==============================================================================
 *函数名称：Get_ADC_Value
 *函数功能：读取某一规则通道AD值
 *输入参数：ch：规则通道ADC_Channel_x；times：读取次数
 *返回值：无
 *备  注：该函数配置好后，返回的结果是N次后的平均值
 *==============================================================================
 */
uint16_t Get_ADC_Value(uint8_t channel, uint8_t times)
{
	uint32_t temp_val = 0;
	uint8_t t;
	// 设置指定ADC的规则组通道，一个序列，采样时间
	// ADC1,ADC通道,239.5个周期,提高采样时间可以提高精确度
	ADC_RegularChannelConfig(ADC1, channel, 1, ADC_SampleTime_239Cycles5);

	for (t = 0; t < times; t++)
	{
		ADC_SoftwareStartConvCmd(ADC1, ENABLE); // 使能指定的ADC1的软件转换启动功能
		while (!ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC))
			; // 等待转换结束
		temp_val += ADC_GetConversionValue(ADC1);
		delay_ms(5);
	}
	return temp_val / times;
}
