#include "stm32f10x.h"                  // Device header
#include "sys.h"
#include "delay.h"
#include "Serial.h"
#include "string.h"
#include "inv_mpu.h"
#include "OLED.h"
#include "IIC.h"
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "event_groups.h"               // ARM.FreeRTOS::RTOS:Event Groups
#include "semphr.h"                     // ARM.FreeRTOS::RTOS:Core
#include "cmsis_os.h"
#define PI 3.141592

// mpu6050    SCL SDA PB6  PB7
// OLED       SCL SDA PB10 PB11

/* 互斥量句柄 */
SemaphoreHandle_t  xMutex;

typedef struct MPU6050_Value
{  
	float Pitch;
	float Roll;
	float Yaw;
} MPU6050_Value; // 创建结构体存储数据名称

typedef struct MAX30102_Value
{  
	uint8_t dis_hr;
	uint8_t dis_spo2;

} MAX30102_Value; // 创建结构体存储数据名称

float Pitch, Roll, Yaw;

void Get_MPU6050_Value(void *pvParameters);
void Change_MPU6050_Value(void *pvParameters);
void Get_MAX30102_Value(void *pvParameters);
void Change_MAX30102_Value(void *pvParameters);

QueueHandle_t Value_queue;
QueueHandle_t Max30102_Value_queue;
int main(void)
{
	delay_init();	 // 延迟函数初始化
	Serial_Init();	//串口初始化 
	OLED_Init(); // OLED初始化
	OLED_Clear();
	mpu_dmp_init(); // MPU6050 DMP初始化

	Value_queue = xQueueCreate(10, sizeof(MPU6050_Value)); // 创建消息队列
	Max30102_Value_queue = xQueueCreate(10, sizeof(MAX30102_Value)); // 创建消息队列
	xMutex = xSemaphoreCreateMutex();
	if( xMutex != NULL )
	{
		xTaskCreate(Get_MPU6050_Value,"Get_MPU6050_Value",1024 * 4 / sizeof(portSTACK_TYPE),NULL,1,NULL); // 创建数据采集任务
		xTaskCreate(Change_MPU6050_Value,"Change_MPU6050_Value",1024 * 4 / sizeof(portSTACK_TYPE),NULL,1,NULL); // 创建数据转换和发送任务
		
		
		xTaskCreate(Get_MAX30102_Value,"Get_MAX30102_Value",1024 * 4 / sizeof(portSTACK_TYPE),NULL,1,NULL); // 创建数据采集任务
		xTaskCreate(Change_MAX30102_Value,"Change_MAX30102_Value",1024 * 4 / sizeof(portSTACK_TYPE),NULL,1,NULL); // 创建数据转换和发送任务

		vTaskStartScheduler(); // 开启任务调度
	}
	else
	{
		/* 无法创建互斥量 */
	}

	/* 如果程序运行到了这里就表示出错了, 一般是内存不足 */
	return 0;
}

void Get_MPU6050_Value(void *pvParameters)
{
	MPU6050_Value Get_value;
	Get_value.Yaw = 0;
	for (;;)
	{
		mpu_dmp_get_data(&Pitch, &Roll, &Yaw); // 读取姿态数据并转换
		Get_value.Pitch = Pitch;
		Get_value.Roll = Roll;
		Get_value.Yaw = Yaw;
		xQueueSend(Value_queue, &Get_value, 0); // 发送数据到消息队列
	}
}

void Change_MPU6050_Value(void *pvParameters)
{
	MPU6050_Value MPU6050_Data;
	for (;;)
	{
		// 如果消息队列中存在数据，那将数据进行转换并发送
		if (pdPASS == xQueueReceive(Value_queue, &MPU6050_Data, portMAX_DELAY))
		{		
			/* 获得互斥量: 上锁 */
			xSemaphoreTake(xMutex, portMAX_DELAY);		
		
			OLED_ShowSignedNum(0, 0, MPU6050_Data.Pitch, 3, OLED_8X16); // OLED显示姿态数据
			OLED_ShowSignedNum(0, 16, MPU6050_Data.Roll, 3, OLED_8X16);
			OLED_ShowSignedNum(0, 32, MPU6050_Data.Yaw, 3, OLED_8X16);	
			OLED_Update();	
		
			/* 释放互斥量: 开锁 */
			xSemaphoreGive(xMutex);
			vTaskDelay(100);  
			

		}
	}
}

void Get_MAX30102_Value(void *pvParameters)
{
	MAX30102_Value Get_value;
	
	for (;;)
	{
		Get_value.dis_hr = Serial_RxPacket[0];
		Get_value.dis_spo2 = Serial_RxPacket[1]; 
		
		xQueueSend(Max30102_Value_queue, &Get_value, 0); // 发送数据到消息队列
	}
}

void Change_MAX30102_Value(void *pvParameters)
{
	MAX30102_Value Max30102_Data;

	for (;;)
	{
		// 如果消息队列中存在数据，那将数据进行转换并发送
		if (pdPASS == xQueueReceive(Max30102_Value_queue, &Max30102_Data, portMAX_DELAY))
		{
					/* 获得互斥量: 上锁 */
			xSemaphoreTake(xMutex, portMAX_DELAY);
		
		
			OLED_ShowSignedNum(64, 0, Max30102_Data.dis_hr, 3, OLED_8X16); // OLED显示姿态数据
			OLED_ShowSignedNum(64, 16, Max30102_Data.dis_spo2, 3, OLED_8X16);		
			OLED_Update();    
			
			/* 释放互斥量: 开锁 */
			xSemaphoreGive(xMutex);
			           			
			vTaskDelay(100);

		}
	}
}


//下面的代码可以成功运行

//#include "stm32f10x.h"                  // Device header
//#include "FreeRTOS.h"
//#include "task.h"
//#include "queue.h"
//#include "string.h"
//#include "event_groups.h"               // ARM.FreeRTOS::RTOS:Event Groups
//#include "semphr.h"                     // ARM.FreeRTOS::RTOS:Core
//#include "cmsis_os.h"

//#include "OLED.h"

//void vSenderTask1( void *pvParameters );
//void vSenderTask2( void *pvParameters );
///* 互斥量句柄 */
//SemaphoreHandle_t  xMutex;
//int main( void )
//{
//	OLED_Init();
//    /* 创建互斥量 */
//	xMutex = xSemaphoreCreateMutex();
//	if( xMutex != NULL )
//	{
//		/* 创建2个任务: 都是打印
//		 * 优先级相同
//		 */
//		xTaskCreate( vSenderTask1, "Sender1", 1000, NULL, 1, NULL );
//		xTaskCreate( vSenderTask2, "Sender2", 1000, NULL, 1, NULL );

//		/* 启动调度器 */
//		vTaskStartScheduler();
//	}
//	else
//	{
//		/* 无法创建互斥量 */
//	}

//	/* 如果程序运行到了这里就表示出错了, 一般是内存不足 */
//	return 0;
//}
//void vSenderTask1( void *pvParameters )
//{
//	uint8_t i = 0;
//	/* 无限循环 */
//	for( ;; )
//	{	
//		i++;
//		OLED_ShowNum(81,0,i,3,OLED_8X16);
//		OLED_ShowString(0,0,"Task1",OLED_8X16);
//		/* 获得互斥量: 上锁 */
//		xSemaphoreTake(xMutex, portMAX_DELAY);
//		
//		
//		OLED_Update();
//		
//		/* 释放互斥量: 开锁 */
//		 xSemaphoreGive(xMutex);
//		vTaskDelay(100);
//	}
//}
//void vSenderTask2( void *pvParameters )
//{
//	uint8_t i = 0;
//	/* 无限循环 */
//	for( ;; )
//	{	
//		i++;
//		OLED_ShowNum(81,16,i,3,OLED_8X16);
//		OLED_ShowString(0,16,"Task2",OLED_8X16);
//		/* 获得互斥量: 上锁 */
//		xSemaphoreTake(xMutex, portMAX_DELAY);
//		
//		
//		OLED_Update();                                
//		/* 释放互斥量: 开锁 */
//		xSemaphoreGive(xMutex);
//		vTaskDelay(100);
//		
//	}
//}












